# _*_coding:utf-8_*_
from django.conf import settings
from django.conf.urls import static
from django.conf.urls import url
from django.urls import re_path

from .apis.resource_image_list import UploadImageList
from .apis.resource_upload_file import UploadFile
from .apis.resource_upload_image import UploadImage
from .apis.resource_upload_video import UploadVideo

# 应用名称
app_name = 'resource'

urlpatterns = [
    url(r'upload_file/?$', UploadFile.as_view(), name='st_upload_file'),  # 文件上传
    url(r'upload_image/?$', UploadImage.as_view(), name='st_upload_image'),  # 图片上传
    url(r'upload_video/?$', UploadVideo.as_view(), name='st_upload_image'),  # 图片上传

    url(r'image_list/?$', UploadImageList.as_view(), name='st_upload_image'),  # 图片上传
    # re_path(r'^_upload_image/?$', UploadImage.as_view(), ),

]



from rest_framework.response import Response
from rest_framework.views import APIView

from xj_user.services.user_service import UserService
from ..services.resource_upload_service import ResourceUploadService
from ..services.resource_video_service import ResourceVideoService


class UploadVideo(APIView):
    def post(self, request):
        # ========== 一、检查：验证权限 ==========
        # TODO 以后改成装饰器，by sieyoo
        token = self.request.META.get('HTTP_AUTHORIZATION', '')
        if not token:
            return Response({'err': 6001, 'msg': '缺少Token', })
        user_serv, error_text = UserService.check_token(token)
        if error_text:
            return Response({'err': 6002, 'msg': error_text, })
        user_id = user_serv.get('user_id', None)
        if not user_id:
            return Response({'err': 7001, 'msg': "用户不存在", })

        # ========== 二、检查：必填性 ==========
        # 对应postman的Body的key=file，value=上传文件的名称 watermelonhh.jpg
        input_file = request.FILES.get("video")
        title = request.POST.get("title")
        group_id = request.POST.get('group_id', None)
        print("> UploadImage: user_id, title, group_id:", user_id, title, group_id)
        if input_file is None:
            return Response({'err': 2001, 'msg': '未选择上传图片', 'tip': '未选择上传图片', })

        # ========== 三、检查：内容的准确性 ==========
        upload_serv = ResourceUploadService(upload_type='video')
        file_info, error_text = upload_serv.init(input_file, by_month=True, user_id=user_id, title=title,
                                                 group_id=group_id, limit_size=None)
        if error_text:
            return Response({'err': 4005, 'msg': error_text, })

        # 写入磁盘
        upload_serv.write(target='disk')
        # print("> UploadImage: upload_serv:", upload_serv)

        # 写入数据库
        video_instance, error_text = ResourceVideoService.add(params=file_info)
        print("> UploadVideo: video_instance:", video_instance, type(video_instance))
        if error_text:
            return Response({'err': 4006, 'msg': error_text, })
        if video_instance:
            file_info['id'] = video_instance.id

        video_dict, error_text = ResourceVideoService.get(video_id=video_instance.id)
        if error_text:
            return Response({'err': 4007, 'msg': error_text, })

        return Response({'err': 0, 'msg': 'OK', 'data': video_dict})



# coding=utf-8
import base64
import hashlib
import os
from pathlib import Path
import random
import re
import time

from main.settings import BASE_DIR
# from ..models import ResourceImageGroup
from ..utils.digit_algorithm import DigitAlgorithm
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict

module_dir = Path(__file__).parent.parent
root_config = JDict(JConfig.get_section(str(BASE_DIR) + '/config.ini', 'xj_resource', encode='utf-8-sig'))
module_config = JDict(JConfig.get_section(str(module_dir) + '/config.ini', 'xj_resource', encode='utf-8-sig'))
if not BASE_DIR:
    print("MSA ERROR: BASE_DIR is not find, check setting.py please!")


class ResourceUploadService:
    """
    @class 资源上传服务。
    @description 该服务只提供文件上传和生成信息，不提供数据库写入！
    """
    # 公共成员，基本数据
    user_id = None  # 用户ID
    title = None  # 文件标题
    filename = None  # 文件名
    url = None  # 文件地址
    suffix = None  # 文件类型
    size = None  # 文件尺寸
    thumb = None  # 缩略图(Base64)
    md5 = None  # MD5校验
    group_id = None  # 文件分组
    counter = None  # 计数器
    # 公共成员，允许类外编辑
    origin_title = None  # 原标题
    origin_filename = None  # 原文件名
    save_dir = None  # 保存目录
    pseudo_dir = None  # 伪静态目录

    # 私有成员，仅限类内查看
    # 配置组
    __root_config = root_config  # 根配置，第一配置，
    __module_config = module_config  # 模块配置，第二配置
    __upload_dir = None  # 上传文件的写入目录。真实目录，相对项目路径，因为大部分情况下，并不希望用户看到文件存放的真实路径
    __upload_url = None  # 上传文件的链接地址。虚拟目录，相对链接路径，因为大部分情况下，并不希望用户看到文件存放的真实路径
    __upload_host = None  # 上传文件的主机
    __upload_format = None  # 支持的文件格式列表
    __upload_limit = None  # 上传文件大小限制，单位：字节

    __file_info = {}  # 文件 信息
    __input_file = None  # 上传文件
    __file_stream = None  # 上传文件流

    __save_absolute_dir = None  # 保存文件的绝对目录
    __save_absolute_file_path = None  # 完整文件绝对路径，包含文件名

    # 构造函数，导入配置文件
    def __init__(self, upload_type='image'):
        """
        @param upload_type 上传文件类型。可选值：image, file, video。不同类型将分别上传至不同的文件夹。
        @description 待补充
        """
        c1 = self.__root_config
        c2 = self.__module_config

        if upload_type.strip().lower() == 'image':
            self.__upload_dir = c1.image_upload_dir or c2.image_upload_dir or "/media/upload/images/"
            self.__upload_url = c1.image_upload_url or c2.image_upload_url or "/media/images/"
            self.__upload_host = c1.image_host or c2.image_host or ""
            self.__upload_formats = c1.image_formats or c2.image_formats or None
            self.__upload_prefix = c1.image_prefix or c2.image_prefix or 'img_'
            self.__upload_limit = c1.image_limit or c2.image_limit or None
            self.__upload_absolute_dir = c1.image_upload_absolute_dir or c2.image_upload_absolute_dir or None

        if upload_type.strip().lower() == 'file':
            self.__upload_dir = c1.file_upload_dir or c2.file_upload_dir or "/media/upload/files/"
            self.__upload_url = c1.file_upload_url or c2.file_upload_url or "/media/files/"
            self.__upload_host = c1.file_host or c2.file_host or ""
            self.__upload_formats = c1.file_formats or c2.file_formats or None
            self.__upload_prefix = c1.image_prefix or c2.image_prefix or 'file_'
            self.__upload_limit = c1.file_limit or c2.file_limit or None
            self.__upload_absolute_dir = c1.file_upload_absolute_dir or c2.file_upload_absolute_dir or None

        if upload_type.strip().lower() == 'video':
            self.__upload_dir = c1.file_upload_dir or c2.file_upload_dir or "/media/upload/videos/"
            self.__upload_url = c1.file_upload_url or c2.file_upload_url or "/media/videos/"
            self.__upload_host = c1.file_host or c2.file_host or ""
            self.__upload_formats = c1.file_formats or c2.file_formats or None
            self.__upload_prefix = c1.image_prefix or c2.image_prefix or 'video_'
            self.__upload_limit = c1.file_limit or c2.file_limit or None
            self.__upload_absolute_dir = c1.video_upload_absolute_dir or c2.video_upload_absolute_dir or None

    # 初始化，基本信息、目录、文件等，初始化完成后可直接写入
    def init(self, input_file, by_month=False, user_id=None, title=None, group_id=None, limit_size=None):
        """
        初始化，基本信息、目录、文件等，初始化完成后可直接写入
        @param input_file {FILE} 输入文件。来自表单提交的FILE
        @param by_month {boolean} 按月分组存储。
        @param user_id {int} 用户ID。
        @param title {string} 文件标题。
        @param group_id {int} 分组ID。
        @param limit_size {int} 文件限制大小。
        @return file_info, error_text {tuple}
        @rtype tuple
        @type FIlE
        @note 待填写
        @author Ian.Sieyoo.Zhao
        @license MSI
        @contact sieyoo@163.com
        @version V1.2.0.20220828
        """
        if not input_file:
            return None, "请选择文件"
        # print("> init: __input_file:", input_file, type(input_file))

        # 先检查目录
        save_dir, error_text = self.init_dir(by_month=by_month)
        if error_text:
            return None, error_text
        # print("> init: save_dir:", save_dir)

        # 再检查文件
        self.__input_file = input_file
        self.user_id = user_id  # 用户ID
        self.title = title  # 文件标题
        self.group_id = group_id  # 分组ID
        self.__upload_limit = limit_size  # 上传限制大小，单位：字节
        file_info, error_text = self.init_file(file_title=title)
        # print("> init: file_info", file_info)
        if error_text:
            return None, error_text
        # print("> init: validate", file_info)

        return self.__file_info, None

    # 读取文件存放的目录，并根据by_month判断是否需要按月份做文件夹分组
    def init_dir(self, directory=None, by_month=False):
        """
        读取文件存放的目录，并根据by_month判断是否需要按月份做文件夹分组
        @param directory {string} 本地相对目录。相对于项目根目录。
        @param by_month {boolean} 按月分组存储。如果分组目录不存在则自动创建。
        @return save_dir, error_text {tuple} 文件目录，错误码
        @description 待填写
        """
        month = time.strftime('%Y-%m', time.localtime(time.time()))
        save_dir = directory
        pseudo_dir = self.__upload_url
        if not save_dir:
            save_dir = self.__upload_dir
        if by_month:
            # 注：添加对多余路径分隔符/\检查
            save_dir = re.sub(r"[/\\]{1,2}", "/", f"/{save_dir}/{month}/")
            pseudo_dir = re.sub(r"[/\\]{1,2}", "/", f"/{pseudo_dir}/{month}/")

        # 如果配置了绝对路径则使用绝对路径，否则使用相对项目路径
        save_absolute_dir = re.sub(r"[/\\]{1,3}", "/", f"{str(BASE_DIR)}/{save_dir}")
        if self.__upload_absolute_dir:
            save_absolute_dir = self.__upload_absolute_dir = self.__upload_absolute_dir + month if by_month else ''
        # 不存在则创建目录
        if not os.path.exists(save_absolute_dir):
            os.makedirs(save_absolute_dir)
        # 再次检查目录存在
        if not os.path.exists(save_absolute_dir):
            return None, "目录创建失败，请检查是否有目录权限：" + save_absolute_dir

        # print("> init_dir: save_dir:", save_dir)
        # print("> init_dir: save_absolute_dir:", save_absolute_dir)
        self.save_dir = save_dir
        self.pseudo_dir = pseudo_dir
        self.__save_absolute_dir = save_absolute_dir
        return save_dir, None

    # 验证文件是否合法
    def init_file(self, file_title=None):
        """
        验证文件是否合法。
        @param file_title {string} 文件标题。修改文件生成的标题。
        @return file_info, error_text {tuple} 文件目录，错误码
        @description 验证文件是否合法，同时生成文件配置信息
        """
        # 一检查文件名。
        self.origin_filename = self.__input_file.name
        ret = re.search(r'(.*)\.(\w{3,5})$', self.__input_file.name)
        if not ret:
            return None, '上传的文件名错误，格式后辍应为3-5个字符'
        self.origin_title = ret.group(1)
        self.suffix = ret.group(2)

        # 三检查文件流
        self.__file_stream = self.__input_file.read()
        if not self.__file_stream:
            return None, '文件流读取失败'
        self.md5 = self.get_md5()

        # 生成唯一的文件名
        self.filename = f"{self.__upload_prefix}{DigitAlgorithm.make_unicode_16(self.md5)}.{self.suffix}"
        # 注：添加对多余路径分隔符/\检查
        self.__save_absolute_file_path = re.sub(r"[/\\]{1,3}", "/", f"{str(BASE_DIR)}/{self.save_dir}/{self.filename}")
        if self.__upload_absolute_dir:
            self.__save_absolute_file_path = re.sub(r"[/\\]{1,3}", "/", f"{self.__upload_absolute_dir}/{self.filename}")
        # print("> init_file: save_absolute_dir:", self.save_absolute_dir)

        # 二检查文件格式。注：添加多余空格检查
        # print("> init_file __upload_format_list:", self.__upload_formats.replace(' ', '').split(","))
        if self.__upload_formats and self.__upload_formats.strip() != '*' and self.suffix \
                and self.suffix not in self.__upload_formats.replace(' ', '').split(","):
            return None, '上传文件格式错误，支持：' + self.__upload_formats

        # 四检查文件大小
        self.size = size = len(self.__file_stream)
        # print("> init_file: __file_stream", type(self.__file_stream), size)
        if size == 0:
            return None, "空文件"
        if self.__upload_limit and size > self.__upload_limit:
            return None, f"文件大小超过限制，不应大于{self.__upload_limit}字节。"

        # 五生成文件信息
        # 本类只处理文件的I/O功能，不处理和数据库相关的读写
        # if self.group_id:
        #     check_group_id = ResourceImageGroup.objects.filter(id=self.group_id)
        #     if not check_group_id:
        #         self.group_id = None
        # self.__file_info['group_id'] = self.group_id
        self.__file_info['user_id'] = self.user_id
        self.__file_info['title'] = self.title = file_title if file_title else self.origin_title
        self.__file_info['filename'] = self.filename
        self.__file_info['url'] = self.url = f"{self.__upload_host}{self.pseudo_dir}{self.filename}"
        self.__file_info['format'] = self.suffix
        self.__file_info['size'] = self.size
        self.__file_info['thumb'] = None
        self.__file_info['md5'] = self.md5
        self.__file_info['snapshot'] = {
            'origin_filename': self.origin_filename,
            'upload_host': self.__upload_host,  # 当代码移植后，域名发生变化，可以通过上传主机查找替换为新主机
            'upload_url': self.__upload_url,  # 上传的地址,由配置产生的伪URL，不含文件夹分组的目录
            'pseudo_dir': self.pseudo_dir,  # 当代码移植后，引用伪目录发生变化，可以通过伪目录替换为新伪目录
            'save_dir': self.save_dir,  # TODO 建议不放快照，因为大部分情况下，并不希望用户看到文件存放的真实路径
            'save_absolute_dir': self.__save_absolute_dir,  # TODO 建议不放快照，真实绝对路径
            'save_absolute_file_path': self.__save_absolute_file_path,  # TODO 建议不放快照，真实绝对路径
        }
        self.__file_info['counter'] = 1
        # print("> init_file: __file_info:", self.__file_info)
        return self.__file_info, None

    def get_md5(self):
        """
        获取文件MD5校验码。
        @return md5 {string} MD5校验码
        @description 略
        """
        content_md5 = hashlib.md5()
        content_md5.update(self.__file_stream)
        content_base64 = base64.b64encode(content_md5.digest())
        return content_base64.decode("utf-8")

    def write(self, target='disk'):
        """
        写入文件
        @return target {string} 写入目标。可选值：disk, oss。
        @description 略
        """
        if target == 'disk':
            self.write_disk()
        if target == 'oss':
            self.write_oss()

    def write_disk(self):
        try:
            # 文件写入磁盘
            # print("> write_disk: save_absolute_path:", self.__save_absolute_file_path)
            with open(self.__save_absolute_file_path, 'wb') as f:
                f.write(self.__file_stream)
        except Exception as e:
            # print("> write_disk Exception:", e)
            pass

    def write_oss(self, config=None):
        # 写入云存储
        pass



# coding=utf-8
from django.db import models
from django.utils import timezone


# ================================== 图片模块 模型=============================
class ResourceImageGroup(models.Model):
    class Meta:
        db_table = u'resource_image_group'
        verbose_name = '3.资源 - 图片分组表'
        verbose_name_plural = verbose_name

    group = models.CharField(verbose_name='图片组名', max_length=128, blank=True, null=True)
    title = models.CharField(verbose_name='图片组标题', max_length=128, blank=True, null=True)
    description = models.CharField(verbose_name='图片组描述', max_length=225, blank=True, null=True)
    source_table = models.CharField(verbose_name='关联数据源表名', max_length=128, blank=True, null=True)


class ResourceImage(models.Model):
    """ 1、Resource_Image 图片表 [NF1] """
    class Meta:
        db_table = u'resource_image'
        verbose_name = '1.资源 - 图片表'
        verbose_name_plural = verbose_name

    user_id = models.BigIntegerField(verbose_name='用户ID', db_column='user_id', db_index=True, blank=True, null=True, help_text='')
    group = models.ForeignKey(ResourceImageGroup, verbose_name='图片分组', db_column='group_id', blank=True, null=True, on_delete=models.DO_NOTHING, help_text='')
    title = models.CharField(verbose_name='图片标题', max_length=255, blank=True, null=True, help_text='')
    url = models.CharField(verbose_name='图片链接', max_length=255, blank=True, null=True, db_index=True, help_text='')
    filename = models.CharField(verbose_name='文件名', max_length=255, blank=True, null=True, help_text='')
    format = models.CharField(verbose_name='文件类型', max_length=10, help_text='')
    size = models.IntegerField(verbose_name='文件尺寸', blank=True, null=True, help_text='')
    thumb = models.TextField(verbose_name='缩略图', blank=True, null=True, help_text='Base64')
    md5 = models.CharField(verbose_name='MD5校验', max_length=255, blank=True, null=True, help_text='')  # 判断使文件是否有效且唯一。
    snapshot = models.JSONField(verbose_name='文件快照', blank=True, null=True, help_text='')  # for Python 3.8
    created_at = models.DateTimeField(verbose_name='创建时间', default=timezone.now, help_text='')
    updated_at = models.DateTimeField(verbose_name='编辑时间', default=timezone.now, help_text='')
    counter = models.IntegerField(verbose_name='使用计数器', blank=True, null=True, help_text='')

    def __str__(self):
        return self.url

    @staticmethod
    def insert(data):
        try:
            res = ResourceImage.objects.create(**data)
            return res.id, None
        except Exception as e:
            return [], '写入异常' + str(e)
            # return [], '写入异常' + e.message


class ResourceImageMap(models.Model):
    class Meta:
        db_table = 'resource_image_map'
        verbose_name_plural = '2. 资源 - 图片映射表'

    image_id = models.ForeignKey(verbose_name='图片ID', to=ResourceImage, related_name='image_id_set+',
                                 on_delete=models.DO_NOTHING, db_column='image_id')
    source_id = models.BigIntegerField(verbose_name='来源ID', blank=True, null=True)
    source_table = models.CharField(verbose_name='来源表', max_length=128, blank=True, null=True)
    price = models.DecimalField(verbose_name='价格', max_digits=32, decimal_places=8, blank=True, null=True,
                                db_index=True)

    def __str__(self):
        return self.image_id


# ================================== 视频模块 模型=============================

class ResourceVideoGroup(models.Model):
    class Meta:
        db_table = u'resource_video_group'
        verbose_name = '资源 - 视频分组表'
        verbose_name_plural = verbose_name

    group = models.CharField(verbose_name='视频组名', max_length=128, blank=True, null=True)
    title = models.CharField(verbose_name='视频组标题', max_length=128, blank=True, null=True)
    description = models.CharField(verbose_name='视频组描述', max_length=225, blank=True, null=True)
    source_table = models.CharField(verbose_name='关联数据源表名', max_length=128, blank=True, null=True)


class ResourceVideo(models.Model):
    class Meta:
        db_table = u'resource_video'
        verbose_name = '资源 - 视频表'
        verbose_name_plural = verbose_name

    group = models.ForeignKey(ResourceVideoGroup, verbose_name='视频分组', db_column='group_id', blank=True, null=True,
                              on_delete=models.DO_NOTHING)
    user_id = models.BigIntegerField(verbose_name='用户ID', db_column='user_id', db_index=True, blank=True, null=True)
    title = models.CharField(verbose_name='视频标题', max_length=255, blank=True, null=True)
    # url = models.ImageField(verbose_name='缩略图', upload_to="static/images", max_length=21845, blank=True, null=True, help_text='缩略图')
    url = models.CharField(verbose_name='视频链接', max_length=255, blank=True, null=True, db_index=True)
    filename = models.CharField(verbose_name='视频名', max_length=255, blank=True, null=True)
    format = models.CharField(verbose_name='视频类型', max_length=32)
    size = models.IntegerField(verbose_name='视频尺寸', blank=True, null=True)
    thumb = models.TextField(verbose_name='缩略图(Base64)', blank=True, null=True)
    md5 = models.CharField(verbose_name='MD5校验', max_length=255, blank=True, null=True)  # 判断使文件是否有效且唯一。
    snapshot = models.JSONField(verbose_name='文件快照', blank=True, null=True)  # for Python 3.8
    created_at = models.DateTimeField(verbose_name='创建时间', default=timezone.now)
    updated_at = models.DateTimeField(verbose_name='编辑时间', default=timezone.now)
    counter = models.IntegerField(verbose_name='使用计数器', blank=True, null=True)

    def __str__(self):
        return self.url



class ResourceVideoMap(models.Model):
    class Meta:
        db_table = 'resource_video_map'
        verbose_name_plural = '资源 - 视频映射表'

    video_id = models.ForeignKey(verbose_name='视频ID', to=ResourceVideo, related_name='video_id_set+',
                                 on_delete=models.DO_NOTHING, db_column='video_id')
    source_id = models.BigIntegerField(verbose_name='来源ID', blank=True, null=True)
    source_table = models.CharField(verbose_name='来源表', max_length=128, blank=True, null=True)
    price = models.DecimalField(verbose_name='价格', max_digits=32, decimal_places=8, blank=True, null=True,
                                db_index=True)

    def __str__(self):
        return self.video_id


# ================================== 文件模块 模型=============================

class ResourceFileGroup(models.Model):
    class Meta:
        db_table = u'resource_file_group'
        verbose_name = '资源 - 文件分组表'
        verbose_name_plural = verbose_name

    group = models.CharField(verbose_name='文件组名', max_length=128, blank=True, null=True)
    title = models.CharField(verbose_name='文件组标题', max_length=128, blank=True, null=True)
    description = models.CharField(verbose_name='文件组描述', max_length=225, blank=True, null=True)
    source_table = models.CharField(verbose_name='关联数据源表名', max_length=128, blank=True, null=True)


class ResourceFile(models.Model):
    class Meta:
        db_table = u'resource_file'
        verbose_name = '资源 - 文件表'
        verbose_name_plural = verbose_name

    group = models.ForeignKey(ResourceFileGroup, verbose_name='文件分组', db_column='group_id', blank=True, null=True,
                              on_delete=models.DO_NOTHING)
    user_id = models.BigIntegerField(verbose_name='用户ID', db_column='user_id', db_index=True, blank=True, null=True)
    title = models.CharField(verbose_name='文件标题', max_length=255, blank=True, null=True)
    # url = models.ImageField(verbose_name='缩略图', upload_to="static/images", max_length=21845, blank=True, null=True, help_text='缩略图')
    url = models.CharField(verbose_name='文件链接', max_length=255, blank=True, null=True, db_index=True)
    filename = models.CharField(verbose_name='文件名', max_length=255, blank=True, null=True)
    format = models.CharField(verbose_name='文件类型', max_length=32)
    size = models.IntegerField(verbose_name='文件尺寸', blank=True, null=True)
    thumb = models.TextField(verbose_name='缩略图(Base64)', blank=True, null=True)
    md5 = models.CharField(verbose_name='MD5校验', max_length=255, blank=True, null=True)  # 判断使文件是否有效且唯一。
    snapshot = models.JSONField(verbose_name='文件快照', blank=True, null=True)  # for Python 3.8
    created_at = models.DateTimeField(verbose_name='创建时间', default=timezone.now)
    updated_at = models.DateTimeField(verbose_name='编辑时间', default=timezone.now)
    counter = models.IntegerField(verbose_name='使用计数器', blank=True, null=True)

    def __str__(self):
        return self.url



class ResourceFileMap(models.Model):
    class Meta:
        db_table = 'resource_file_map'
        verbose_name_plural = '资源 - 文件映射表'

    file_id = models.ForeignKey(verbose_name='文件ID', to=ResourceFile, related_name='file_id_set+',
                                on_delete=models.DO_NOTHING, db_column='file_id')
    source_id = models.BigIntegerField(verbose_name='来源ID', blank=True, null=True)
    source_table = models.CharField(verbose_name='来源表', max_length=128, blank=True, null=True)
    price = models.DecimalField(verbose_name='价格', max_digits=32, decimal_places=8, blank=True, null=True,
                                db_index=True)

    def __str__(self):
        return self.file_id



# coding=utf-8
import base64
import hashlib
import os
import re

from rest_framework import serializers

from .. import config
from ..models import ResourceFile
from ..utils.digit_algorithm import DigitAlgorithm


# 声明序列化，处理处理数据并写入数据
class UploadFileSerializer(serializers.ModelSerializer):
    class Meta:
        model = ResourceFile
        # 序列化验证检查，检查必填项的字段
        fields = ['id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot']


# 用于异常处理
def robust(actual_do):
    def add_robust(*args, **keyargs):
        try:
            return actual_do(*args, **keyargs)
        except Exception as e:
            print(str(e))

    return add_robust


class UploadFileService(object):
    base_path = config.BASE_DIR  # 根目录
    folder_path = config.FILE_UPLOAD_DIR  # 存放路径，不含文件名
    save_path = None  # 完整上传路径，包含文件名
    filename = None  # 重新命名
    input_file = None  # 上传文件
    file_info = {}  # 文件 信息

    old_filename = None
    new_filename = None
    suffix = None

    __is_valid = True  # 是否存在异常
    __error_message = None  # 异常消息

    def __init__(self, input_file):
        self.input_file = input_file
        self.file_code = input_file.read()
        self.base_path = str(self.base_path)
        self.validate()

    def validate(self):
        # 验证文件是否合法
        if self.input_file is None:
            self.__set_error('请选择文件')
            return False
        ret = re.search(r'(.*)\.(\w{3,4})$', self.input_file.name)
        if not ret:
            self.__set_error('上传的文件名不合法')
            return False

        self.old_filename = ret.group(1)
        self.suffix = ret.group(2)
        if config.file_format_list:
            if not self.suffix in config.file_format_list:
                self.__set_error('上传的文件类型不合法')
                return False

    def get_md5(self):
        content_md5 = hashlib.md5()
        content_md5.update(self.file_code)
        content_base64 = base64.b64encode(content_md5.digest())
        return content_base64.decode("utf-8")

    def write_disk(self):
        try:
            if not self.__is_valid:
                return False
            # 文件写入磁盘
            path = self.base_path + "/" + self.folder_path
            if not os.path.exists(path):
                os.makedirs(path)
            with open(self.save_path, 'wb') as f:
                f.write(self.file_code)
        except Exception as e:
            self.__set_error(str(e))

    def write_oss(self, config):
        # 写入云存储
        pass

    def save_to_db(self, save_data):
        # 素材入库保存
        try:
            if not self.__is_valid:
                return False
            serializer = UploadFileSerializer(data=save_data, context={})
            if not serializer.is_valid():
                return [], serializer.errors
            image_instance = serializer.save()
            return image_instance
        except Exception as e:
            self.__set_error(str(e))
            return False

    def info_detail(self):
        try:
            """获取文件信息并返回"""
            self.new_filename = 'file_' + DigitAlgorithm.make_unicode_16() + "." + self.suffix
            self.save_path = self.base_path + self.folder_path + self.new_filename
            # 获取MD5
            self.file_info = {
                'url': (self.folder_path + self.new_filename).replace('//', '/'),
                'filename': self.new_filename,
                'format': self.suffix,
                'md5': self.get_md5(),
                'snapshot': {
                    'old_filename': self.old_filename,
                    'suffix': self.suffix
                }
            }
            return self.file_info
        except Exception as e:
            self.__set_error(str(e))
            return {}

    # ============ 异常处理 =================
    def __set_error(self, error_message):
        if not self.__error_message:
            self.__error_message = error_message
        self.__is_valid = False

    def is_valid(self):
        return self.__is_valid

    def get_error_message(self):
        return self.__error_message



# coding=utf-8
import os

from rest_framework import serializers

from ..models import ResourceImage


# 用于异常处理
def robust(actual_do):
    def add_robust(*args, **keyargs):
        try:
            return actual_do(*args, **keyargs)
        except Exception as e:
            print(str(e))

    return add_robust


# # 声明序列化，处理处理数据并写入数据
# class UploadImageSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = ResourceImage
#         # 序列化验证检查，检查必填项的字段
#         fields = ['id', 'group', 'group_id', 'user_id', 'title', 'url', 'filename', 'format', 'size', 'thumb', 'md5', 'snapshot',
#                   'counter']
#
#
# class ResourceImageService:
#
#     def __init__(self):
#         pass
#
#     @staticmethod
#     def create(params):
#         serializer = UploadImageSerializer(data=params)
#         if not serializer.is_valid():
#             return None, serializer.errors
#
#         instance = serializer.save()
#         return instance, None


class ResourceImageService:

    def __init__(self):
        pass

    @staticmethod
    def get(image_id):
        image_set = ResourceImage.objects.filter(id=image_id).values(
            "id",
            "group_id",
            "user_id",
            "title",
            "filename",
            "url",
            "format",
            "size",
            "thumb",
            "md5",
            "snapshot",
            "counter",
        ).first()
        if not image_set:
            return None, '数据库找不到图片'
        return image_set, None

    @staticmethod
    def add(params):
        # print("> ResourceImageService create:", params)
        # # 检查MD5是否有历史上传记录，暂不使用，因为目录按月份分组时这些数据变得不稳定
        # md5 = params.get('md5', None)
        # if md5 is None:
        #     return False
        # instance = ResourceImage.objects.filter(md5=md5).first()
        # if instance:
        #     return instance, None

        # sieyoo注，数据写入最好通过显式的方式，这样可以随时修改和扩展复杂的需求，代价是必须和模型一致，另一个好处是开发者加深数据库结构的记忆
        new_params = {
            'group_id': params.get('group_id', None),
            'user_id': params.get('user_id', None),
            'title': params.get('title', None),
            'url': params.get('url', None),
            'filename': params.get('filename', None),
            'format': params.get('format', None),
            'size': params.get('size', None),
            'thumb': params.get('thumb', None),
            'md5': params.get('md5', None),
            'snapshot': params.get('snapshot', None),
            'counter': params.get('counter', 1),
        }

        try:
            image_set = ResourceImage(**new_params)
            image_set.save()
        except Exception as e:
            return None, f"""图片保存失败，请检查数据库。 {str(e)} in "{str(e.__traceback__.tb_frame.f_globals["__file__"])}" : Line {str(e.__traceback__.tb_lineno)}"""

        return image_set, None



# encoding: utf-8
"""
@project: hydrology-station-python-4.0->validate
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 表单验证基类
@created_time: 2022/5/25 11:43
"""
import json

from django import forms
from django.core.exceptions import ValidationError


class Validate(forms.Form):
    """检验基类，子类编写规则，调用父类的validate方法"""

    def validate(self):
        """
        request 请求参数验证
        :return {'code': 'err': self.errors}:
        """
        if self.is_valid():
            return True, None
        else:
            error = json.dumps(self.errors)
            error = json.loads(error)
            temp_error = {}
            # 统一展示小写 提示，中文转义回来
            for k, v in error.items():
                temp_error[k.lower()] = v[0]
            return False, temp_error


# 案例使用自定义验证方法
def log_unit_id(value):
    res = True
    if not res:
        raise ValidationError('unit_id不存在')


class ExampleValidate(Validate):
    """验证查询表单"""
    region_code = forms.CharField(
        required=True,
        error_messages={
            "required": "行政编码 必填",
        })
    unit_id = forms.IntegerField(  # 自定义方法
        validators=[log_unit_id],
    )



from django.contrib import admin

from config.config import Config
from .models import ResourceImage, ResourceImageMap, ResourceFile, ResourceFileMap, ResourceVideo, ResourceVideoMap

# Register your models here.
config = Config.getIns()
admin.site.site_header = config.get('main', 'app_name', 'msa一体化管理后台')
admin.site.site_title = config.get('main', 'app_name', 'msa一体化管理后台')


# =============================== 图片 ==========================================
class ResourceImageAdmin(admin.ModelAdmin):
    fields = ('id', 'user_id', 'group', 'title', 'url', 'filename', 'format', 
              'size', 'thumb', 'md5', 'snapshot', 'created_at', 'updated_at', 'counter')
    list_display = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')
    search_fields = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')
    readonly_fields = ['id']


class ResourceImageMapAdmin(admin.ModelAdmin):
    fields = ('id', 'image_id', 'source_id', 'source_table', 'price',)
    list_display = ('id', 'image_id', 'source_id', 'source_table', 'price',)
    search_fields = ('id', 'image_id', 'source_id', 'source_table', 'price',)


# =============================== 文件 ==========================================
class ResourceFileAdmin(admin.ModelAdmin):
    fields = ('user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot', 'thumb')
    list_display = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')
    search_fields = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')


class ResourceFileMapAdmin(admin.ModelAdmin):
    fields = ('id', 'file_id', 'source_id', 'source_table', 'price',)
    list_display = ('id', 'file_id', 'source_id', 'source_table', 'price',)
    search_fields = ('id', 'file_id', 'source_id', 'source_table', 'price',)


# =============================== 视频 ==========================================
class ResourceVideoAdmin(admin.ModelAdmin):
    fields = ('user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot', 'thumb')
    list_display = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')
    search_fields = ('id', 'user_id', 'title', 'url', 'filename', 'format', 'md5', 'snapshot')


class ResourceVideoMapAdmin(admin.ModelAdmin):
    fields = ('id', 'video_id', 'source_id', 'source_table', 'price',)
    list_display = ('id', 'video_id', 'source_id', 'source_table', 'price',)
    search_fields = ('id', 'video_id', 'source_id', 'source_table', 'price',)


# =============================== end ==========================================
admin.site.register(ResourceImage, ResourceImageAdmin)
admin.site.register(ResourceImageMap, ResourceImageMapAdmin)

admin.site.register(ResourceFile, ResourceFileAdmin)
admin.site.register(ResourceFileMap, ResourceFileMapAdmin)

admin.site.register(ResourceVideo, ResourceFileAdmin)
admin.site.register(ResourceVideoMap, ResourceVideoMapAdmin)



from django.core.paginator import Paginator
from rest_framework.views import APIView

from ..models import *
from ..utils.model_handle import util_response, only_filed_handle, parse_model


class UploadImageList(APIView):
    # 文件列表
    def get(self, request):
        params = request.query_params.copy()
        limit = params.pop('limit', 20)
        page = params.pop('page', 20)
        params = only_filed_handle(params, {
            "title": "title__contains",
            "filename": "filename_contains",
            "md5": "md5",
            "user_id": "user_id"
        }, None)
        list_obj = ResourceImage.objects.filter(**params)
        count = list_obj.count()
        res_set = Paginator(list_obj, limit).get_page(page)
        return util_response(data={'count': count, 'page': page, 'limit': limit, "list": parse_model(res_set)})



# encoding: utf-8
"""
@project: resource->config
@author: 孙楷炎
@synopsis: 文件模块配置文件
@created_time: 2022/5/28 12:19
"""

import os
import time
# from pathlib import Path  # for Python 3.8

from main.settings import STATIC_URL, BASE_DIR

# resource模块应放在apps.resource目录下
# BASE_DIR = Path(__file__).resolve().parent.parent.parent # for Python 3.8 项目根目录
# BASE_DIR =os.path.abspath(os.path.join(os.getcwd(), '.'))  # 项目根目录

today = time.strftime('%Y-%m', time.localtime(time.time()))
FILE_UPLOAD_DIR = STATIC_URL + "/upload/file/" + today + "/"
IMAGE_UPLOAD_DIR = STATIC_URL + "/upload/image/" + today + "/"

# file_format_list:  False  # false 不限制文件类型 TODO 逻辑宏常量应该转成大写
file_format_list = False
img_format_list = ('jpg', 'jpeg', 'gif', 'png', 'svg')
host = ''

# 资源文件路径。注：此路径需包含在settings.py的STATICFILES_DIRS中
RESOURCE_FILES_DIR = os.path.join(BASE_DIR, "resource_files")



# _*_coding:utf-8_*_

from django.apps import AppConfig


class ResourceConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'xj_resource'
    verbose_name = u'Ⅵ 资源系统'
    sort = 6
